from flask import jsonify, request
from flask_cors import cross_origin
from flask_expects_json import expects_json
from flask_jwt_extended import jwt_required

from areas import api_v1
from helpers.auth_guard import admin_required

from .tag_service import TagService
from .validation import schema


@api_v1.route("/tags", methods=["GET"])
@jwt_required()
@cross_origin()
@admin_required()
def get_tags():
    result = TagService.get_tags()
    return jsonify(result)

@api_v1.route("/tags", methods=["POST"])
@jwt_required()
@cross_origin()
@expects_json(schema)
@admin_required()
def post_tag():
    data = request.get_json()
    TagService.create_tag(data)
    return jsonify(message="Tag created successfully.")

@api_v1.route("/tags/<int:id>", methods=["PUT"])
@jwt_required()
@cross_origin()
@expects_json(schema)
@admin_required()
def put_tag(id):
    data = request.get_json()
    result = TagService.update_tag(id, data)
    return jsonify(message="Tag updated successfully.")

@api_v1.route("/tags/<int:id>", methods=["DELETE"])
@jwt_required()
@cross_origin()
@admin_required()
def delete_tag(id):
    TagService.delete_tag(id)
    return jsonify(message="Tag deleted successfully.")
