import re

schema = {
    "type": "object",
    "properties": {
        "email": {
            "type": "string",
            "description": "Email of the user",
            "pattern": r"(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|\"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*\")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])",
            "minLength": 1,
        },
        "app_roles": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "name": {
                        "type": "string",
                        "description": "Name of the app",
                        "minLength": 1,
                    },
                    "role_id": {
                        "type": ["integer", "null"],
                        "description": "Role of the user",
                        "minimum": 1,
                    },
                },
                "required": ["name", "role_id"],
            },
        },
        "tags": {
            "type": "array",
            "items": {
                "type": "integer",
            },
        },
    },
    "required": ["email", "app_roles"],
}

schema_multiple = {
    "users": {
        "type": "array",
        "items": {
            "$ref": schema
        }
    }
}

# Multiple app role edit of existing users
schema_multi_edit = {
    "users": {
        "type": "array",
        "items" : {
            "type": "object",
            "properties": {
                "email": {
                    "type": "string",
                    "description": "Email of the user",
                    "minLength": 1,
                },
                "id": {
                    "type": "string",
                    "description": "UUID of the user",
                    "minLength": 1,
                },
                "app_roles": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "properties": {
                            "name": {
                                "type": "string",
                                "description": "Name of the app",
                                "minLength": 1,
                            },
                            "role_id": {
                                "type": ["integer", "null"],
                                "description": "Role of the user",
                                "minimum": 1,
                            },
                        },
                        # "required": ["name", "role_id"],
                    },
                },
                "tags": {
                    "type": "array",
                    "items": {
                        "type": "integer",
                    },
                },
            },
            # "required": ["email", "app_roles"],
        }
    }
}

schema_recovery_complete = {
    "type": "object",
    "properties": {
        "user_id": {
            "type": "string",
            "description": "Kratos ID of the user",
            "minLength": 1,
        },
    },
    "required": ["user_id"],
}
