from flask import current_app

from areas.apps.models import App, AppRole
from areas.roles.models import Role
from config import *
from database import db

def user_has_access(user, app):
    # Get role on dashboard
    dashboard_app = db.session.query(App).filter(
        App.slug == 'dashboard').first()
    if not dashboard_app:
        current_app.logger.error("Dashboard app not found in database.")
        return False
    role_object = (
        db.session.query(AppRole)
        .filter(AppRole.app_id == dashboard_app.id)
        .filter(AppRole.user_id == user.uuid)
        .first()
    )
    if role_object is None:
        current_app.logger.info(f"No dashboard role set for user {user.uuid}.")
        return False

    # If the user is dashboard admin, they have access to everything.
    if role_object.role_id == Role.ADMIN_ROLE_ID:
        current_app.logger.info(f"User {user.uuid} has admin dashboard role")
        return True

    # Get role for app.
    role_object = (
        db.session.query(AppRole)
        .filter(AppRole.app_id == app.id)
        .filter(AppRole.user_id == user.uuid)
        .first()
    )
    # Role ID 3 is always "No access" due to migration b514cca2d47b
    if role_object is None or role_object.role_id is None or role_object.role_id == Role.NO_ACCESS_ROLE_ID:
        current_app.logger.info(f"User {user.uuid} has no access for: {app.name}")
        return False

    # In all other cases, access is granted.
    return True
