from functools import wraps
import os

from areas.roles.role_service import RoleService
from helpers import Unauthorized

from flask import request
from flask_jwt_extended import get_jwt, verify_jwt_in_request


def admin_required():
    def wrapper(fn):
        @wraps(fn)
        def decorator(*args, **kwargs):
            verify_jwt_in_request()
            claims = get_jwt()
            user_id = claims["user_id"]
            is_admin = RoleService.is_user_admin(user_id)
            if is_admin:
                return fn(*args, **kwargs)
            else:
                raise Unauthorized("You need to have admin permissions.")

        return decorator

    return wrapper

def kratos_webhook():
    def wrapper(fn):
        @wraps(fn)
        def decorator(*args, **kwargs):
            header = request.headers.get("Authorization")
            if header is not None and header == os.environ.get("KRATOS_WEBHOOK_SECRET"):
                return fn(*args, **kwargs)
            else:
                raise Unauthorized("This needs a valid api key.")

        return decorator

    return wrapper
